//*****************************************************************************
//
// blinky.c - Simple example to blink the on-board LED.
//
// Copyright (c) 2009-2010 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of the Micromint Eagle Firmware Package.
//
//*****************************************************************************

#if defined(BOARD_EAGLE50)
#include "inc/lm3s1918.h"
#elif defined(BOARD_EAGLE60E)
#include "inc/lm3s9b90.h"
#else
// Eagle 50E or Eagle 100
#include "inc/lm3s6918.h"
#endif

#if defined(BOARD_EAGLE60E)
// LED (PD0)
#define LED_PORT_SYSCTL         SYSCTL_RCGC2_GPIOD
#define LED_PORT_DATA_R         GPIO_PORTD_DATA_R
#define LED_PORT_DIR_R          GPIO_PORTD_DIR_R
#define LED_PORT_DEN_R          GPIO_PORTD_DEN_R
#define LED_PIN                 0x01
#else
// LED (PE1)
#define LED_PORT_SYSCTL         SYSCTL_RCGC2_GPIOE
#define LED_PORT_DATA_R         GPIO_PORTE_DATA_R
#define LED_PORT_DIR_R          GPIO_PORTE_DIR_R
#define LED_PORT_DEN_R          GPIO_PORTE_DEN_R
#define LED_PIN                 0x02
#endif

//*****************************************************************************
//
//! \addtogroup example_list
//! <h1>Blinky (blinky)</h1>
//!
//! A very simple example that blinks the on-board LED.
//
//*****************************************************************************

//*****************************************************************************
//
// Blink the on-board LED.
//
//*****************************************************************************
int
main(void)
{
    volatile unsigned long ulLoop;

    //
    // Enable the GPIO port that is used for the on-board LED.
    //
    SYSCTL_RCGC2_R = LED_PORT_SYSCTL;

    //
    // Do a dummy read to insert a few cycles after enabling the peripheral.
    //
    ulLoop = SYSCTL_RCGC2_R;

    //
    // Enable the GPIO pin for the LED (PE1).  Set the direction as output, and
    // enable the GPIO pin for digital function.
    //
    LED_PORT_DIR_R = LED_PIN;
    LED_PORT_DEN_R = LED_PIN;

    //
    // Loop forever.
    //
    while(1)
    {
        //
        // Turn on the LED.
        //
        LED_PORT_DATA_R |= LED_PIN;

        //
        // Delay for a bit.
        //
        for(ulLoop = 0; ulLoop < 200000; ulLoop++)
        {
        }

        //
        // Turn off the LED.
        //
        LED_PORT_DATA_R &= ~(LED_PIN);

        //
        // Delay for a bit.
        //
        for(ulLoop = 0; ulLoop < 200000; ulLoop++)
        {
        }
    }
}
