/*
  monitor.c - Monitor I/O functions (Cortex-M3)
              by Micromint Support <support@micromint.com>    Jan-2009

  Uses monitor.svf firmware on CPLD

  Copyright (c) 2008-2010 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "monitor.h"

/* -------------------- Monitor functions -------------------- */
/* Initialize monitor */
void MonitorInit(void)
{
    XioInit();
}

/* Read monitor counter */
unsigned long MonitorCountGet(void)
{
    return XioRead(MONITOR_PORT);
}

/* Write monitor counter */
unsigned long MonitorCountSet(unsigned long data)
{
    return XioWrite(MONITOR_PORT, data);
}

/* Read monitor limit */
unsigned long MonitorLimitGet(void)
{
    return XioRead(MONITOR_LIMIT);
}

/* Write monitor limit */
unsigned long MonitorLimitSet(unsigned long data)
{
    return XioWrite(MONITOR_LIMIT, data);
}

/* Send monitor command */
unsigned long MonitorCommand(unsigned long data)
{
    return XioWrite(MONITOR_CMD, data);
}

/* Enable monitor interrupt when alarm pin (PA7) is asserted */
void MonitorIntEnable(void)
{
    /*
       Configure the interrupt line as an input and
         enable the pin to interrupt on the falling edge
    */
    SysCtlPeripheralEnable(MONITOR_SYSCTL_PERIPH);
    GPIOPinTypeGPIOInput(MONITOR_GPIO_BASE, MONITOR_ALARM_PIN);
    GPIOPadConfigSet(MONITOR_GPIO_BASE, MONITOR_ALARM_PIN, GPIO_STRENGTH_2MA,
                     GPIO_PIN_TYPE_STD_WPU);
    GPIOIntTypeSet(MONITOR_GPIO_BASE, MONITOR_ALARM_PIN, GPIO_FALLING_EDGE);
    GPIOPinIntEnable(MONITOR_GPIO_BASE, MONITOR_ALARM_PIN);
    IntEnable(MONITOR_INT_GPIO);
}

/* Disable monitor interrupt */
void MonitorIntDisable(void)
{
    IntDisable(MONITOR_INT_GPIO);
    GPIOPinIntDisable(MONITOR_GPIO_BASE, MONITOR_ALARM_PIN);
}

/* Clear monitor interrupt */
void MonitorIntClear(void)
{
    /* Clear GPIO interrupt */
    GPIOPinIntClear(MONITOR_GPIO_BASE, MONITOR_ALARM_PIN);
}
