/*
  dac_ex.c - Tests the DAC
              by Micromint Support <support@micromint.com>    May-2009

  Copyright (c) 2008-2009 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "inc/hw_gpio.h"
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "driverlib/debug.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/ssi.h"
#include "driverlib/sysctl.h"
#include "driverlib/systick.h"
#include "utils/uartstdio.h"
#include "dac.h"

//Global Variable
static unsigned long digital_count = 0; // Holds the digital count for the DAC
static char chrCount = 1;     // Tells which direction to increment the count

/*
  Handler for SysTick interrupt
*/
void SysTickHandler(void)
{
    DacWrite(DAC0, digital_count);
    if (digital_count>=1023) chrCount = 0;
    else if (digital_count==0) chrCount = 1;
    if (chrCount)   digital_count = digital_count + 1;
    if (!(chrCount)) digital_count = digital_count - 1;
}

/*
  Main function to dac example
*/
int main(void)
{
    /* Adjust LDO to 2.75V according to REV A2 errata when using PLL */
    SysCtlLDOSet(SYSCTL_LDO_2_75V);
    /* Set the clock to run at 50 MHz using the PLL */
    SysCtlClockSet(SYSCTL_SYSDIV_4| SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);
    /* --- Enable UART --- */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);
    /* Intialize the DAC */
    InitDac();
    DacWrite(DAC1, DAC_VOLT(1.5)); /*Set channel 1 to 1.5V */
    DacWrite(DAC2, DAC_VOLT(2.0)); /*Set channel 2 to 2.0V */
    DacWrite(DAC3, DAC_VOLT(2.5)); /*Set channel 3 to 2.5V */
    
    /* Test DAC */
    UARTprintf("DAC Test\n");
    
    /* SysTick is a 24-bit count-down timer - max is 16,777,216 (0x01000000)
    Using a 50 MHz processor clock the period is 20 ns. In order to create
    a triangle wave the system tick for the DAC the following formula should be
    used:
    ticks = 1/F/2048/(1/sysclock)
    There are 2048 intervals per cycle at 20nS per interval sys tick needs to 
    interrupt the controller
    75 Hz - 326
    100 Hz - 244 Tick Counts
    150 Hz - 162 Tick Counts */ 
    SysTickPeriodSet(244);
    SysTickEnable();
    IntRegister(FAULT_SYSTICK, SysTickHandler);
    SysTickIntEnable();
    /* Loop while waiting for interrupts */    
    while(1)
    {
    }      
}
