/*
  rtc_ex.c - Tests I2C Real-Time Clock Calender
           by Micromint Support <support@micromint.com>    Nov-2009

 Copyright (c) 2009 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the disclaimer below.
  2.  Micromint's name may not be used to endorse or promote products derived
      from this software without specific prior written permission.
  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY MICROMINT "AS IS" AND ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
  DISCLAIMED. IN NO EVENT SHALL MICROMINT BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_nvic.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_types.h"
#include "driverlib/debug.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/hibernate.h"
#include "driverlib/uart.h"
#include "utils/uartstdio.h"
#include "utils/cmdline.h"
#include "rtc.h"
#include "delay.h"
//*****************************************************************************
//
// Defines the size of the buffer that holds the command line.
//
//*****************************************************************************
#define CMD_BUF_SIZE    64

//*****************************************************************************
//
// The buffer that holds the command line.
//
//*****************************************************************************
static char g_cCmdBuf[CMD_BUF_SIZE];
//*****************************************************************************
//
// Array for the days of the week.
//
//*****************************************************************************
const unsigned char day_week[7][10]={"Sunday",
                                     "Monday",
                                     "Tuesday",
                                     "Wednesday",
                                     "Thursday",
                                     "Friday",
                                     "Saturday"};
//*****************************************************************************
//
// This function handles the RTC alarm interrupts
//
//*****************************************************************************

void intHibernateHandler(void)
{
  unsigned char cmd_status1;
  unsigned char cmd_status2;
  unsigned char cmd_status2_copy;
  if(HibernateIntStatus(0) & HIBERNATE_INT_PIN_WAKE)
  {
    UARTprintf("\nRTC Alarm Happend\n");
    /* Read the second status register to save what interrupt is enabled */
    RtcRdRegister(&cmd_status2, CMD_STATUS2);
    cmd_status2_copy = cmd_status2;
    /* Disable the alarm on the RTC */
    cmd_status2_copy &= ~(FLAG_INT1ME | FLAG_INT1FE | FLAG_INT1AE);
    RtcWrRegister(&cmd_status2_copy, CMD_STATUS2);
    /* Wait for the pin to go high */
    delay_ms(8);
    /* Read status 1 regiter to clear INT1 flag */
    RtcRdRegister(&cmd_status1, CMD_STATUS1);
    /* Turn the alarm back on */
    RtcWrRegister(&cmd_status2, CMD_STATUS2);
    HibernateIntClear(HIBERNATE_INT_PIN_WAKE);
  }
  else
  {
    UARTprintf("Some other hibernate interrupt has occurred\n");
  }
}

//*****************************************************************************
//
// This function validates a date and time.
//
//*****************************************************************************
unsigned long ValidDateTime(struct rtc_time *tm, unsigned long format)
{
  if(tm->tm_sec > 59) return 0;
  if(tm->tm_min > 59) return 0;
  if((tm->tm_hour > 24) && (format == 24)) return 0;
  if(((tm->tm_hour < 1) || (tm->tm_hour > 12)) && (format == 12)) return 0;
  if((tm->tm_mday < 1) || (tm->tm_mday > 31)) return 0;
  if((tm->tm_mon < 1) || (tm->tm_mon > 12)) return 0;
  if((tm->tm_year < 1) || (tm->tm_year > 99)) return 0;
  return 1; /* date is valid */
}
//*****************************************************************************
//
// This function validates a time.
//
//*****************************************************************************
unsigned long ValidTime(struct rtc_time *tm, unsigned long format)
{
  if(tm->tm_min > 59) return 0;
  if((tm->tm_hour > 24) && (format == 24)) return 0;
  if(((tm->tm_hour < 1) || (tm->tm_hour > 12)) && (format == 12)) return 0;
  return 1; /* date is valid */
}
//*****************************************************************************
//
// This function seperates an the hours and minutes string into a binary time.
//
//*****************************************************************************
void string_hrmn(char *argv,struct rtc_time *altm)
{
  char temp[2];
  //Convert the hours
  temp[0]= argv[0];
  temp[1]= argv[1];
  altm->tm_hour = atoi(temp);
  //Convert the minutes
  temp[0]= argv[3];
  temp[1]= argv[4];
  altm->tm_min = atoi(temp);
}
//*****************************************************************************
//
// This function implements the "chfrmt" command.  It changes the time format
// if the time format is 24 hour before entering this routine it will be 12 hour
// after leaving.
//
//*****************************************************************************
int Cmd_chfrmt(int argc, char *argv[])
{
    unsigned long format;
    format = RtcChngTimeFormat();
    if (format == 12) UARTprintf("Time is in 12 hour mode.\n");
    else UARTprintf("Time is in 24 hour mode.\n");
    return(0);
}

//*****************************************************************************
//
// This function implements the "rddt" command.  It displays the current
// date and time that the Real-Time Clock Calender is set to.
//
//*****************************************************************************
int Cmd_rddt(int argc, char *argv[])
{
    struct rtc_time tm;
    int intSec = 0;
    unsigned long format;
    unsigned char timeout=0;
    format = RtcTimeFormat();
    UARTprintf("Press any key to continue\n");
    RtcReadDateTime(&tm);
    intSec = tm.tm_sec;
    while (!(UARTCharsAvail(UART0_BASE)))
    {
      UARTprintf("%s %02d/%02d/%02d %02d:%02d:%02d",day_week[tm.tm_wday],
                                                    tm.tm_mon,tm.tm_mday,tm.tm_year,
                                                    tm.tm_hour ,tm.tm_min,tm.tm_sec);
      if (format == 12)
      {
        if (tm.tm_ampm) UARTprintf(" PM\r");
        else UARTprintf(" AM\r");
      }
      else UARTprintf("\r");

      while((intSec == tm.tm_sec) && (timeout < 2))
      {
        RtcReadDateTime(&tm);
        delay_ms(900);
        timeout++; // This is in case the alarm interrupts the reading
      }
      timeout = 0 ;
      intSec = tm.tm_sec;
    }
    UARTCharGet(UART0_BASE); //clear the anykey
    return(0);
}

//*****************************************************************************
//
// This function implements the "setdt" command.
//
//*****************************************************************************
int Cmd_setdt(int argc, char *argv[])
{
  struct rtc_time tm;
  char temp[2];
  unsigned long format;
  format = RtcTimeFormat();
  if (format == 12)
  {
     if (!(strncmp(argv[4],"am",2))) tm.tm_ampm = 0x0;
     else if (!(strncmp(argv[4],"pm",2))) tm.tm_ampm = 0x1;
     else
     {
       UARTprintf("Invalid am/pm. '%s' was entered.\n Needs to be lowercase am or pm.\n",argv[4]);
       return(0);
     }
  }
  // Set the day of the week
  if(!(strncmp(argv[1],"sun",3))) tm.tm_wday = 0x0;
  else if(!(strncmp(argv[1],"mon",3))) tm.tm_wday = 0x1;
  else if(!(strncmp(argv[1],"tue",3))) tm.tm_wday = 0x2;
  else if(!(strncmp(argv[1],"wed",3))) tm.tm_wday = 0x3;
  else if(!(strncmp(argv[1],"thu",3))) tm.tm_wday = 0x4;
  else if(!(strncmp(argv[1],"fri",3))) tm.tm_wday = 0x5;
  else if(!(strncmp(argv[1],"sat",3))) tm.tm_wday = 0x6;
  else
  {
    UARTprintf("Invalid day of the week. String '%s' was entered.\nNeeds to be lowercase sun, mon, tue, wed, thu, fri, or sat.\n",argv[1]);
    return(0);
  }
  //Convert the month
  temp[1]= argv[2][1];
  temp[0]= argv[2][0];
  tm.tm_mon = atoi(temp);
  //Convert the day of month
  temp[1]= argv[2][4];
  temp[0]= argv[2][3];
  tm.tm_mday = atoi(temp);
  //Convert the year
  temp[1]= argv[2][7];
  temp[0]= argv[2][6];
  tm.tm_year = atoi(temp);
  //Convert the hours
  temp[1]= argv[3][1];
  temp[0]= argv[3][0];
  tm.tm_hour = atoi(temp);
  //Convert the minutes
  temp[1]= argv[3][4];
  temp[0]= argv[3][3];
  tm.tm_min = atoi(temp);
  //Convert the seconds
  temp[1]= argv[3][7];
  temp[0]= argv[3][6];
  tm.tm_sec = atoi(temp);

  if (!(ValidDateTime(&tm, format)))
  {
    UARTprintf("Invalid date and time entered. '%s %s' was entered.\n",argv[2],argv[3]);
    UARTprintf("Leading zeros are required and be sure that you entered it in %d hour format.\n",format);
    UARTprintf("Enter date in format ##/##/## and time in format ##:##:##.\n");
    return(0);
  }
  RtcSetDateTime(&tm);
  RtcReadDateTime(&tm); // This starts the clock incrementing
  UARTprintf("%s %02d/%02d/%02d %02d:%02d:%02d",day_week[tm.tm_wday],
                                                    tm.tm_mon,tm.tm_mday,tm.tm_year,
                                                    tm.tm_hour ,tm.tm_min,tm.tm_sec);
  if (format == 12)
  {
    if (tm.tm_ampm) UARTprintf(" PM\n");
    else UARTprintf(" AM\n");
  }
  else UARTprintf("\n");
  // Clear the arguments
  strcpy(argv[1]," ");
  strcpy(argv[2]," ");
  strcpy(argv[3]," ");
  strcpy(argv[4]," ");
  return(0);
}
//*****************************************************************************
//
// This function implements the "setdt" command.
//
//*****************************************************************************
int Cmd_setdtex(int argc, char *argv[])
{
  UARTprintf("The following commands will set the date and time to ");
  UARTprintf("Wednesday 11/25/09 1:00 PM.\n");
  UARTprintf("24 Hour Mode: setdt wed 11/25/09 13:00:00\n");
  UARTprintf("12 Hour Mode: setdt wed 11/25/09 01:00:00 pm\n");
  return(0);
}
//*****************************************************************************
//
// This function implements the "tmfrmt" command. It displays what format the
// time is in.
//
//*****************************************************************************
int Cmd_tmfrmt(int argc, char *argv[])
{
  unsigned long format;
  format = RtcTimeFormat();
  if (format == 12) UARTprintf("Time is in 12 hour mode.\n");
  else UARTprintf("Time is in 24 hour mode.\n");
  return(0);
}

//*****************************************************************************
//
// This function implements the "alroff" command. It turns the alarm off.
//
//*****************************************************************************
int Cmd_alroff(int argc, char *argv[])
{
  // Disable the interrupt on the microcontroller
  HibernateIntDisable(HIBERNATE_INT_PIN_WAKE);
  RtcDisableAlarm();

  return(0);
}
//*****************************************************************************
//
// This function implements the "setalr" command. It sets the alarm
//
//*****************************************************************************
int Cmd_setalr(int argc, char *argv[])
{
  struct rtc_time altm;
  char temp[2];
  unsigned long format;
  unsigned long occurrence;

  format = RtcTimeFormat();
  // Process arugment 1
  if(!(strncmp(argv[1],"permin",6)))occurrence = AL_PERMIN;
  else if(!(strncmp(argv[1],"weekly",6))) occurrence = AL_WEEKLY;
  else if(!(strncmp(argv[1],"daily",5))) occurrence = AL_DAILY;
  else if(!(strncmp(argv[1],"hourly",6))) occurrence = AL_HOURLY;
  else
  {
    UARTprintf("Invalid alarm occurrence. String '%s' was entered.\nNeeds to be lowercase permin, weekly, daily, or hourly.\n",argv[1]);
    return(0);
  }
  // Process Aurgement 2
  if (occurrence == AL_WEEKLY)
  {
    if(!(strncmp(argv[2],"sun",3))) altm.tm_wday = 0x0;
    else if(!(strncmp(argv[2],"mon",3))) altm.tm_wday = 0x1;
    else if(!(strncmp(argv[2],"tue",3))) altm.tm_wday = 0x2;
    else if(!(strncmp(argv[2],"wed",3))) altm.tm_wday = 0x3;
    else if(!(strncmp(argv[2],"thu",3))) altm.tm_wday = 0x4;
    else if(!(strncmp(argv[2],"fri",3))) altm.tm_wday = 0x5;
    else if(!(strncmp(argv[2],"sat",3))) altm.tm_wday = 0x6;
    else
    {
      UARTprintf("Invalid day of the week. String '%s' was entered.\nNeeds to be lowercase sun, mon, tue, wed, thu, fri, or sat.\n",argv[2]);
      return(0);
    }
  }
  else if (occurrence == AL_DAILY)
  {
    string_hrmn(&argv[2][0],&altm);
    if (!(ValidTime(&altm, format)))
    {
      UARTprintf("Invalid time entered. String '%s' was entered and the time is in %d mode.\n",argv[2], format);
      return(0);
    }
  }
  else //AL_HOURLY
  {
    temp[1]= argv[2][1];
    temp[0]= argv[2][0];
    altm.tm_min = atoi(temp);
    if(altm.tm_min > 59)
    {
      UARTprintf("Invalid minutes entered. String '%s' was entered.\n",argv[2]);
      return 0;
    }
  }

  // Process Argument 3
  if (occurrence == AL_WEEKLY)
  {
    string_hrmn(&argv[3][0],&altm);
    if (!(ValidTime(&altm, format)))
    {
      UARTprintf("Invalid time entered. String '%s' was entered and the time is in %d mode.\n",argv[3], format);
      return(0);
    }
  }
  else if ((occurrence == AL_DAILY) && (format == 12))
  {
    if (!(strncmp(argv[3],"am",2))) altm.tm_ampm = 0x0;
    else if (!(strncmp(argv[3],"pm",2))) altm.tm_ampm = 0x1;
    else
    {
      UARTprintf("Invalid am/pm. '%s' was entered.\n Needs to be lowercase am or pm.\n",argv[3]);
      return(0);
    }
  }
  // Process Argument 4
  if ((occurrence == AL_WEEKLY) && (format == 12))
  {
    if (!(strncmp(argv[4],"am",2))) altm.tm_ampm = 0x0;
    else if (!(strncmp(argv[4],"pm",2))) altm.tm_ampm = 0x1;
    else
    {
      UARTprintf("Invalid am/pm. '%s' was entered.\n Needs to be lowercase am or pm.\n",argv[4]);
      return(0);
    }
  }
  // Enable the alarm
  RtcEnableAlarm(occurrence);
  // Enable the interrupt on the microcontroller
  HibernateIntEnable(HIBERNATE_INT_PIN_WAKE);
  if((occurrence == AL_WEEKLY) || (occurrence == AL_DAILY) || (occurrence == AL_HOURLY))
  {
    // Set the alarm time
    RtcSetAlTime(occurrence,&altm);
  }
  // Clear the arguments
  strcpy(argv[1]," ");
  strcpy(argv[2]," ");
  strcpy(argv[3]," ");
  strcpy(argv[4]," ");
  return(0);
}
//*****************************************************************************
//
// This function implements the "rdalr" command. It displays what the alarm
// is set to.
//
//*****************************************************************************
int Cmd_rdalr(int argc, char *argv[])
{
  struct rtc_time altm;
  unsigned long occurrence;
  unsigned long format;
  unsigned char statusreg2=0;
  format = RtcTimeFormat();
  RtcRdRegister(&statusreg2, CMD_STATUS2);

  if(statusreg2 == FLAG_INT1ME)
  {
    UARTprintf("The alarm occurs every minute.\n");
    return(0);
  }
  else if(statusreg2 == FLAG_INT1AE)
  {
    occurrence = RtcReadAlTime(&altm);
  }
  else
  {
    UARTprintf("The alarm is not set.\n");
    return(0);
  }

  if (occurrence == AL_WEEKLY)
  {
    UARTprintf("at %s %02d:%02d",day_week[altm.tm_wday],altm.tm_hour,altm.tm_min);

    if (format == 12)
    {
       if (altm.tm_ampm) UARTprintf(" PM");
       else UARTprintf(" AM");
     }
  }
  else if (occurrence == AL_DAILY)
  {
    UARTprintf("The alarm occurs daily at %02d:%02d",altm.tm_hour,altm.tm_min);
    if (format == 12)
    {
       if (altm.tm_ampm) UARTprintf(" PM");
       else UARTprintf(" AM");
     }
  }
  else if (occurrence == AL_HOURLY)
  {
    UARTprintf("The alarm occurs hourly when the minutes equal %d",altm.tm_min);
  }
  else
  {
    UARTprintf("Error retrieving alarm time.\n");
    return(0);
  }
  UARTprintf(".\n",argv[4]);
  return(0);
}
//*****************************************************************************
//
// This function implements the "setalex" command.
//
//*****************************************************************************
int Cmd_setalex(int argc, char *argv[])
{
  UARTprintf("The following command will set the alarm to go off every minute.\n");
  UARTprintf("Any Mode: setalr permin\n\n");

  UARTprintf("The following commands will set the alarm to go off every ");
  UARTprintf("Tuesday at 1:30 PM.\n");
  UARTprintf("24 Hour Mode: setalr weekly tue 13:30\n");
  UARTprintf("12 Hour Mode: setalr weekly tue 01:30 pm\n\n");

  UARTprintf("The following commands will set the alarm to go off everyday ");
  UARTprintf("at 1:30 PM.\n");
  UARTprintf("24 Hour Mode: setalr daily 13:30\n");
  UARTprintf("12 Hour Mode: setalr daily 01:30 pm\n\n");

  UARTprintf("The following command will set the alarm to go off everytime ");
  UARTprintf("the minutes are 30 past the hour.\n");
  UARTprintf("Any Mode: setalr hourly 30\n\n");
  return(0);
}
//*****************************************************************************
//
// This function implements the "rdstat" command. It displays both status
// registers for the RTC.
//
//*****************************************************************************
int Cmd_rdstat(int argc, char *argv[])
{
  unsigned char statusreg1=0;
  unsigned char statusreg2=0;
  RtcRdRegister(&statusreg1, CMD_STATUS1);
  RtcRdRegister(&statusreg2, CMD_STATUS2);
  UARTprintf("Status Register 1 = %d, Status Register 2 = %d\n",statusreg1,statusreg2);
  return(0);
}
//*****************************************************************************
//
// This function implements the "help" command.  It prints a simple list
// of the available commands with a brief description.
//
//*****************************************************************************
int Cmd_help(int argc, char *argv[])
{
    tCmdLineEntry *pEntry;

    //
    // Print some header text.
    //
    UARTprintf("\nAvailable commands\n");
    UARTprintf("------------------\n");

    //
    // Point at the beginning of the command table.
    //
    pEntry = &g_sCmdTable[0];

    //
    // Enter a loop to read each entry from the command table.  The
    // end of the table has been reached when the command name is NULL.
    //
    while(pEntry->pcCmd)
    {
        //
        // Print the command name and the brief description.
        //
        UARTprintf("%s%s\n", pEntry->pcCmd, pEntry->pcHelp);

        //
        // Advance to the next entry in the table.
        //
        pEntry++;
    }
    //
    // Return success.
    //
    return(0);
}

//*****************************************************************************
//
// This is the table that holds the command names, implementing functions,
// and brief description.
//
//*****************************************************************************
tCmdLineEntry g_sCmdTable[] =
{
    { "help",   Cmd_help,      "    : Display list of commands" },
    { "h",      Cmd_help,      "       : alias for help" },
    { "?",      Cmd_help,      "       : alias for help" },
    { "chfrmt", Cmd_chfrmt,    "  : Change the time format" },
    { "rddt",   Cmd_rddt,      "    : Display date and time" },
    { "setdt",  Cmd_setdt,     "   : Set the date and time (setdt sun ##/##/## ##:##:## am)" },
    { "setdtex", Cmd_setdtex,  " : Display example commands for setting the date and time"},
    { "tmfrmt", Cmd_tmfrmt,    "  : Check what format the time is in"},
    { "alroff", Cmd_alroff,    "  : Turn the alram off"},
    { "setalr", Cmd_setalr,    "  : Set the alarm (setalr weekly tue ##:## am)"},
    { "setalex", Cmd_setalex,  " : Display example commands for setting the alarm"},
    { "rdalr",  Cmd_rdalr,     "   : Display what the alarm is set to"},
    { "rdstat" , Cmd_rdstat,   "  : Display the RTC status registers"},
    { 0, 0, 0 }
};

//*****************************************************************************
//
// The program main function.  It performs initialization, then runs
// a command processing loop to read commands from the console.
//
//*****************************************************************************
int
main(void)
{
    int nStatus;
    //Adjust LDO to 2.75V according to REV A2 errata when using PLL
    SysCtlLDOSet(SYSCTL_LDO_2_75V);

     /* Set the clock to run at 50 MHz using the PLL */
    SysCtlClockSet(SYSCTL_SYSDIV_4| SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);
    //
    // Enable the peripherals used by this example.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    //
    // Enable interrupts to the processor.
    //
    IntMasterEnable();
    // Set GPIO A0 and A1 as UART.
    //
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    // Initialize the Real-Time Clock
    RtcInit();
    // Initiailaze the Real-Time Clocks Alarm
    SysCtlPeripheralEnable(SYSCTL_PERIPH_HIBERNATE);
    HibernateEnableExpClk(SysCtlClockGet());
    // Choose the 4.194304-MHz crystal
    HibernateClockSelect(HIBERNATE_CLOCK_SEL_DIV128);
    HibernateWakeSet(HIBERNATE_WAKE_PIN);

    // Enable Hibernate interrupt
    HibernateIntRegister(intHibernateHandler);


    //
    // Initialize the UART as a console for text I/O.
    //
    UARTStdioInit(0);

    //
    // Print hello message to user.
    //
    UARTprintf("\n\nReal-Time Clock Calender Program\n");
    UARTprintf("Type \'help\' for help.\n");

    //
    // Enter an infinite loop for reading and processing commands from the
    // user.
    //
    while(1)
    {
        UARTprintf("\nPlease be sure to set the time using the %d hour format.",RtcTimeFormat());
        //
        // Print a prompt to the console.  Show the CWD.
        //
        UARTprintf("\n>");

        //
        // Get a line of text from the user.
        //
        UARTgets(g_cCmdBuf, sizeof(g_cCmdBuf));

        //
        // Pass the line from the user to the command processor.
        // It will be parsed and valid commands executed.
        //
        nStatus = CmdLineProcess(g_cCmdBuf);

        //
        // Handle the case of bad command.
        //
        if(nStatus == CMDLINE_BAD_CMD)
        {
            UARTprintf("Bad command!\n");
        }

        //
        // Handle the case of too many arguments.
        //
        else if(nStatus == CMDLINE_TOO_MANY_ARGS)
        {
            UARTprintf("Too many arguments for command processor!\n");
        }

        //
        // Otherwise the command was executed.  Print the error
        // code if one was returned.
        //
        else if(nStatus != 0)
        {
            UARTprintf("nStatus !=0\n");
        }
    }
}
