/*
  rs232_485.c - Example to switch between RS232 and RS485 on COM2
                of the Micromint Eagle SBC
  Board should have the Plus I/O option installed.
  by Micromint Support <support@micromint.com>    May-2009

  Copyright (c) 2009 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the disclaimer below.
  2.  Micromint's name may not be used to endorse or promote products derived
      from this software without specific prior written permission.
  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY MICROMINT "AS IS" AND ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
  DISCLAIMED. IN NO EVENT SHALL MICROMINT BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "inc/hw_gpio.h"
#include "driverlib/debug.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/uart.h"
#include "delay.h"

#define LED_PIN                 GPIO_PIN_1
#define LED_REG                 HWREG(GPIO_PORTE_BASE + (GPIO_O_DATA + (LED_PIN << 2)))

//*****************************************************************************
//
// The RS232IntHandler interrupt handler.
//
//*****************************************************************************
void
RS232IntHandler(void)
{
    unsigned long ulStatus;
   //
   // Disable the RS-485 port UART to stop the noise due to turning on the
   // transmitter on the driver.
   //
    UARTDisable(UART1_BASE);
    //
    // Enable the RS-485 drivers transmitter
    //
    GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_3,GPIO_PIN_3);
    //
    // Get the interrrupt status.
    //
    ulStatus = UARTIntStatus(UART0_BASE, true);
    //
    // Clear the asserted interrupts.
    //
    UARTIntClear(UART0_BASE, ulStatus);
    //
    // Enable the RS485 UART
    //
    UARTEnable(UART1_BASE);
    //
    // Loop while there are characters in the receive FIFO.
    //
    while(UARTCharsAvail(UART0_BASE))
    {
        //
        // Read the next character from the UART and write it back to the UART.
        //
        UARTCharPutNonBlocking(UART1_BASE, UARTCharGetNonBlocking(UART0_BASE));
    }
    //
    // Disable the RS-485 port UART to stop the noise due to turning off the
    // transmitter on the driver.
    //
    UARTDisable(UART1_BASE);
    //
    // Disable the RS-485 drivers transmitter
    //
    GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_3,0);
     //
    // Enable the RS485 UART
    //
    UARTEnable(UART1_BASE);
}
//*****************************************************************************
//
// The RS485IntHandler interrupt handler.
//
//*****************************************************************************
void
RS485IntHandler(void)
{
    unsigned long ulStatus;

    //
    // Get the interrrupt status.
    //
    ulStatus = UARTIntStatus(UART1_BASE, true);

    //
    // Clear the asserted interrupts.
    //
    UARTIntClear(UART1_BASE, ulStatus);
    //
    // Loop while there are characters in the receive FIFO.
    //
    while(UARTCharsAvail(UART1_BASE))
    {
        //
        // Read the next character from the UART and write it back to the UART.
        //
        UARTCharPutNonBlocking(UART0_BASE, UARTCharGetNonBlocking(UART1_BASE));
    }
}

//*****************************************************************************
//
// Send a string to the UART.
//
//*****************************************************************************
void
UARTSend(const unsigned char *pucBuffer, unsigned long ulCount)
{
    //
    // Loop while there are more characters to send.
    //
    while(ulCount--)
    {
        //
        // Write the next character to the UART.
        //
        UARTCharPutNonBlocking(UART0_BASE, *pucBuffer++);
    }
}

//*****************************************************************************
//
// This example demonstrates a RS232 to RS485 converter.
//
//*****************************************************************************
int
main(void)
{
    // Adjust LDO to 2.75V according to REV A2 errata when using PLL
    SysCtlLDOSet(SYSCTL_LDO_2_75V);
    /* Set the clock to run at 50 MHz using the PLL */
    SysCtlClockSet(SYSCTL_SYSDIV_4| SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);
    //
    // Enable the peripherals used by this example.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    HWREG(GPIO_PORTE_BASE + GPIO_O_DIR) |= LED_PIN;
    HWREG(GPIO_PORTE_BASE + GPIO_O_DEN) |= LED_PIN;
    //
    // Enable processor interrupts.
    //
    IntMasterEnable();

    //
    // Set GPIO A0 and A1 as UART pins.
    //
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    //
    // Set GPIO D2 and D3 as UART pins.
    //
    GPIOPinTypeUART(GPIO_PORTD_BASE, GPIO_PIN_2 | GPIO_PIN_3);
    //
    // Configure the UART for 115,200, 8-N-1 operation.
    //
    UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 115200,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));

    UARTConfigSetExpClk(UART1_BASE, SysCtlClockGet(), 115200,
                        (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                         UART_CONFIG_PAR_NONE));
    //
    // Enable the UART interrupt.
    //
    IntRegister(INT_UART0, RS232IntHandler);
    IntEnable(INT_UART0);
    UARTIntEnable(UART0_BASE, UART_INT_RX | UART_INT_RT);
    IntRegister(INT_UART1, RS485IntHandler);
    IntEnable(INT_UART1);
    UARTIntEnable(UART1_BASE, UART_INT_RX | UART_INT_RT);
    //
    // Make A3 an output to control RS485 Trasmitter.
    //
    GPIOPinTypeGPIOOutput(GPIO_PORTA_BASE, GPIO_PIN_3);
    //
    // Disable the RS-485 drivers transmitter
    //
    GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_3,0);
    //
    // Loop forever echoing data through the UART.
    //
    while(1)
    {
      LED_REG ^= LED_PIN; /* Toggle LED */
      delay_ms(250);
    }
}
