/*
  ansi.c - ANSI screen driver for the Micromint Eagle (Cortex-M3)
          by Micromint Support <support@micromint.com>    Aug-2009

  Uses COM2 (UART1) for I/O

  Copyright (c) 2008-2009 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "ansi.h"

static unsigned long ansiPort;

#define MAX_DIGITS 8
static unsigned char *ansi_ltoa(long, unsigned char *);

/* -------------------- ANSI functions -------------------- */
void AnsiInit(unsigned long port)
{
  /* Initialize UART peripherals */
  switch(port)
  {
    case 0: /* UART0 */
      ansiPort = UART1_BASE;
      SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
      SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
      /* Set PA0 and PA1 as UART pins. */
      GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
      break;
    default: /* UART1 */
      ansiPort = UART1_BASE;
      SysCtlPeripheralEnable(SYSCTL_PERIPH_UART1);
      SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
      /* Set PD2 and PD3 as UART pins. */
      GPIOPinTypeUART(GPIO_PORTD_BASE, GPIO_PIN_2 | GPIO_PIN_3);
      break;
  }
  /* Configure the UART for 115,200, 8-N-1 operation */
  UARTConfigSetExpClk(ansiPort, SysCtlClockGet(), 115200,
                      (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                       UART_CONFIG_PAR_NONE));
  UARTEnable(ansiPort);
  AnsiClrScr();
}

void AnsiClrScr(void)
{
  AnsiPutString("\033[H\033[2J");
}

void AnsiGoTo(unsigned long x, unsigned long y)
{
  AnsiPutString("\033[");
  AnsiPutNum(y+1); //(y);
  AnsiPutChar(';');
  AnsiPutNum(x ? x+1 : x); //(x);
  AnsiPutChar('H');
}

unsigned long AnsiGetChar()
{
  return UARTCharGet(ansiPort);
}

void AnsiPutChar(unsigned char c)
{
  UARTCharPut(ansiPort, c);
}

void AnsiPutString(unsigned char *str)
{
  /* Loop until null is found */
  while(*str != '\0')
  {
    /* Write character to serial port */
    AnsiPutChar(*str++);
  }
}

void AnsiPutNum(long n)
{
  unsigned char s[MAX_DIGITS];
  AnsiPutString(ansi_ltoa(n, s));
}

static unsigned char *ansi_ltoa(long n, unsigned char *p)
{
  int neg;

  p += (MAX_DIGITS - 1);
  *p = '\0';
  if ((neg = (n < 0))) n = -n;

  do {
    *(--p) = (n % 10) + '0';
  } while ((n /= 10) > 0);

  if (neg) *(--p) = '-';

  return p;
}
