/*
  dac.c - Digital to Analog functions for the Micromint Eagle (Cortex-M3)
          by Micromint Support <support@micromint.com>    May-2009

  Copyright (c) 2008-2009 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
  3. Neither the name Micromint nor the names of its contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/
#include "dac.h"

/* -------------------- DAC functions -------------------- */
/* Initialize the DAC */
void InitDac(void)
{
    SysCtlPeripheralEnable(DAC_SSI_SYSCTL_PERIPH);
    SysCtlPeripheralEnable(DAC_GPIO_SYSCTL_PERIPH);
    GPIOPinTypeSSI(DAC_GPIO_PORT_BASE, DAC_SSI_PINS);
    SysCtlPeripheralEnable(DAC_SYNC_GPIO_SYSCTL_PERIPH);
    GPIOPinTypeGPIOOutput(DAC_SYNC_PORT_BASE, DAC_SYNC_PIN);
    GPIOPadConfigSet(DAC_GPIO_PORT_BASE, DAC_SSI_PINS, GPIO_STRENGTH_4MA,
                     GPIO_PIN_TYPE_STD_WPU);
    GPIOPadConfigSet(DAC_SYNC_PORT_BASE, DAC_SYNC_PIN, GPIO_STRENGTH_4MA,
                     GPIO_PIN_TYPE_STD_WPU);
    /* Disable the DAC */
    GPIOPinWrite(DAC_SYNC_PORT_BASE, DAC_SYNC_PIN,DAC_SYNC_PIN_DISABLE);
    /* Freescale SPI frame format mode 0 (polarity=0, phase=0)
         SSI master, 4 MHz clock, 16 data bits
    */
    /* Set the SSI port to half the system clock */
    SSIDisable(DAC_SSI_BASE);
    SSIConfigSetExpClk(DAC_SSI_BASE, SysCtlClockGet(), SSI_FRF_MOTO_MODE_2,
                       SSI_MODE_MASTER, SysCtlClockGet()/2, 16);
    SSIEnable(DAC_SSI_BASE);
}
/* Write to the DAC */
unsigned long DacWrite(unsigned long channel, unsigned long conv_count)
{
    unsigned long rcvData;
    /* Enable the DAC */
    GPIOPinWrite(DAC_SYNC_PORT_BASE, DAC_SYNC_PIN,DAC_SYNC_PIN_ENABLE);
    SSIDataPutNonBlocking(DAC_SSI_BASE, (channel | WR_REG_UPDATE | (conv_count << 2)));
    while(HWREG(DAC_SSI_BASE + SSI_O_SR) & SSI_SR_BSY);
    SSIDataGet(DAC_SSI_BASE, &rcvData); /* Flush received data */
    /*Disable the DAC */
    GPIOPinWrite(DAC_SYNC_PORT_BASE, DAC_SYNC_PIN,DAC_SYNC_PIN_DISABLE);
    return 0;
}
