/*
  dac.h - Digital to Analog Extended  declarations for the 
          Micromint Eagle (Cortex-M3)
          by Micromint Support <support@micromint.com>    May-2009
*/

#ifndef _DAC_H
#define _DAC_H

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "inc/hw_gpio.h"
#include "inc/hw_ssi.h"
#include "driverlib/gpio.h"
#include "driverlib/ssi.h"
#include "driverlib/sysctl.h"

/* GPIO for DAC */
#define DAC_SYNC_PIN                 GPIO_PIN_0      /* DAC Sync GPIO */
#define DAC_SYNC_PIN_ENABLE          0x00
#define DAC_SYNC_PIN_DISABLE         0x01
#define DAC_SYNC_PORT_BASE           GPIO_PORTG_BASE /* DAC SYNC pin port */
#define DAC_SYNC_GPIO_SYSCTL_PERIPH  SYSCTL_PERIPH_GPIOG

/* SSI port */
#define DAC_SSI_BASE              SSI0_BASE
#define DAC_SSI_SYSCTL_PERIPH     SYSCTL_PERIPH_SSI0

/* GPIO for SSI pins - PA2/SSI0CLK, PA4/SSI0RX, PA5/SSI0TX */
#define DAC_GPIO_PORT_BASE        GPIO_PORTA_BASE
#define DAC_GPIO_SYSCTL_PERIPH    SYSCTL_PERIPH_GPIOA
#define DAC_SSI_CLK               GPIO_PIN_2
#define DAC_SSI_TX                GPIO_PIN_5
#define DAC_SSI_PINS              (DAC_SSI_TX | DAC_SSI_CLK)

/*Input Shift Register Difinition for DAC
A1 A0 OP1 OP0 D11 D10 D9 D8 D7 D6 D5 D4 D3 D2 D1 D0
A1, A0 - for addressing 00 - DAC A, 01 - DAC B, 10 - DAC C, 11 - DAC D
OP1, OP0 - Output Instructions
00 - Write to specified register but do not update outputs
01 - Write to specified register and update outputs
10 - Write to all registers and update outputs
11 - Power down outputs 
For further information on Power Down refer to the DAC104085 data sheet 
D2 - D11 are for the 10-bit conversion count. D0 - D1 does not matter

Channel and Output Instruction Declarations */
#define DAC0               0x00
#define DAC1               0x4000
#define DAC2               0x8000
#define DAC3               0xC000
#define WR_REG_UPDATE      0x1000 /*Write to specified register and update outputs */


#define DAC_RES     1024
#define DAC_RANGE   3.3
#define DAC_OFF     0
#define DAC_VOLT(volts) (int)((DAC_RES*(volts+DAC_OFF))/DAC_RANGE)  

/* API function declarations */

/* -------------------- DAC functions -------------------- */
void InitDac(void);
unsigned long DacWrite(unsigned long,unsigned long);
#endif /* _DAC_H */
