/*
  dac.h - Digital to Analog Extended  declarations for the
          Micromint Eagle (Cortex-M3)
          by Micromint Support <support@micromint.com>    May-2009
*/

#ifndef __DELAY_H__
#define __DELAY_H__


//#define CPU_CLK        50000000  // CPU Clock Frequency.
#ifndef CPU_CLK
   #warning "Please define the CPU_CLK clock in hertz before you include delay.h."
#endif
/* CPU Cycle Defines per microsecond, millisecond and second */
#define CPU_CLOCKS_PER_CYCLE    (1)
#define CPU_CYCLES_PER_SEC      (CPU_CLK/CPU_CLOCKS_PER_CYCLE)
#define CPU_CYCLES_PER_US       (CPU_CYCLES_PER_SEC/1000000.0)
#define CPU_CYCLES_PER_MS       (CPU_CYCLES_PER_SEC/1000.0)

// ---------------------------------------------------------------------------
// CONSTANTS AND COMPUTATIONS
// ---------------------------------------------------------------------------

/* Cycles per loop for each delay function  */
#define DELAY_CYCLES_PER_LOOP      (3)

/* Calulate how many CPU clycles per microsecond,
   millisecond and second for one delay loop */
#define DELAY_LOOPS_PER_US   (CPU_CYCLES_PER_US/DELAY_CYCLES_PER_LOOP)
#define DELAY_LOOPS_PER_MS   (CPU_CYCLES_PER_MS/DELAY_CYCLES_PER_LOOP)
#define DELAY_LOOPS_PER_SEC  (CPU_CYCLES_PER_SEC/DELAY_CYCLES_PER_LOOP)

/* Microsecond delay routine */
#define delay_us(us)   delay_loop( ((unsigned long) ( (us) * (DELAY_LOOPS_PER_US) )) )

//* Millisecond delay routine */
#define delay_ms(ms)   delay_loop( ((unsigned long) ( (ms) * (DELAY_LOOPS_PER_MS) )) )

//* Second delay routine */
#define delay_s(s)     delay_loop( ((unsigned long) ( (s) * (DELAY_LOOPS_PER_SEC) )) )

//* Delay loop used for all of the delays */

static void delay_loop(unsigned long loops)
{
   #ifdef ewarm
    asm("loop:               \n"
        "subs.w  R0, R0,#1   \n" /* Decrement counter */
        "bne.w   loop        \n"); /* Loop if not done */
  #else
    __asm__ volatile
    (                                        \
      "\n\t"                                 \
      "LOOP_%=:                        \n\t" \
      "subs %0, %0, #1                 \n\t" \
      "bne LOOP_%=                     \n\t" \
      "\n\t"                                 \
      : "=r" (loops)                         \
      : "0" (loops)                          \
      : "cc"                                 \
    );



  //  __asm__(
  //    "\n\t"
  //    "loop:                     \n"
  //    "subs R0, R0, #1           \n"
  //    "bne loop                  \n"
  //    : "=r" (loops)
  //    : "0" (loops)
  //    : "cc"
//   );


  #endif
}


// ---------------------------------------------------------------------------

#endif
