/*
  rtc.c - Real-Time Clock Calendar functions for the Micromint Eagle (Cortex-M3)
          by Micromint Support <support@micromint.com>    Nov-2009

 Copyright (c) 2009 Micromint USA LLC  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the disclaimer below.
  2.  Micromint's name may not be used to endorse or promote products derived
      from this software without specific prior written permission.
  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY MICROMINT "AS IS" AND ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
  DISCLAIMED. IN NO EVENT SHALL MICROMINT BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "i2c_rdwr.h"

/* -------------------- I2C functions -------------------- */
/* I2C write routine */
unsigned long I2cWrite(unsigned long ulI2CBase,unsigned char address, unsigned char *buf, unsigned char numb_bytes)
{
     unsigned char i;
    /* If the count is less than one then return an error */
     if(numb_bytes < 1)
     {
         return 1; /* Error */
     }
     /* if the count is one then only perform a byte write */
     else if (numb_bytes == 1)
     {
       I2CMasterSlaveAddrSet(ulI2CBase, address, false);
       I2CMasterDataPut(ulI2CBase, *buf);
       I2CMasterControl(ulI2CBase, I2C_MASTER_CMD_SINGLE_SEND);
       while(I2CMasterBusy(ulI2CBase));
       return 0; /* No Errors */
     }
     /* bytes must be greater than 1 */
     else
     {
         for (i = 0; i < numb_bytes; ++i)
         {
            if (i == 0)
            {
              I2CMasterSlaveAddrSet(ulI2CBase, address, false);
              I2CMasterDataPut(ulI2CBase, *buf);
              I2CMasterControl(ulI2CBase, I2C_MASTER_CMD_BURST_SEND_START);
              while(I2CMasterBusy(ulI2CBase));
            }
            else if (i == numb_bytes)
            {
              I2CMasterDataPut(ulI2CBase, *buf);
              I2CMasterControl(ulI2CBase, I2C_MASTER_CMD_BURST_SEND_FINISH);
              while(I2CMasterBusy(ulI2CBase));
            }
            else
            {
              I2CMasterDataPut(ulI2CBase, *buf);
              I2CMasterControl(ulI2CBase, I2C_MASTER_CMD_BURST_SEND_CONT);
              while(I2CMasterBusy(ulI2CBase));
            }
            buf++;
         }
     }
     return 0;
}
/* I2C read routine */
unsigned long I2cRead(unsigned long ulI2CBase, unsigned char address, unsigned char *buf, unsigned char numb_bytes)
{
   unsigned char i;
    /* If the count is less than one then return an error */
     if(numb_bytes < 1)
     {
         return 1; /* Error */
     }
     /* if the count is one then only perform a byte write */
     else if (numb_bytes == 1)
     {
       I2CMasterSlaveAddrSet(ulI2CBase, address, true);
       I2CMasterControl(ulI2CBase, I2C_MASTER_CMD_SINGLE_RECEIVE);
       while(I2CMasterBusy(ulI2CBase));
       *buf = (unsigned char) I2CMasterDataGet(ulI2CBase);
       return 0; /* No Errors */
     }
     /* bytes must be greater than 1 */
     else
     {
         for (i = 0; i < numb_bytes; ++i)
         {
            if (i == 0)
            {
              I2CMasterSlaveAddrSet(ulI2CBase, address, true);
              I2CMasterControl(ulI2CBase, I2C_MASTER_CMD_BURST_RECEIVE_START);
              while(I2CMasterBusy(ulI2CBase));
              *buf = (unsigned char) I2CMasterDataGet(ulI2CBase);
            }
            else if (i == numb_bytes)
            {
              I2CMasterSlaveAddrSet(ulI2CBase, address, true);
              I2CMasterControl(ulI2CBase, I2C_MASTER_CMD_BURST_RECEIVE_FINISH);
              while(I2CMasterBusy(ulI2CBase));
              *buf = (unsigned char) I2CMasterDataGet(ulI2CBase);
            }
            else
            {
              I2CMasterSlaveAddrSet(ulI2CBase, address, true);
              I2CMasterControl(ulI2CBase, I2C_MASTER_CMD_BURST_RECEIVE_CONT);
              while(I2CMasterBusy(ulI2CBase));
              *buf = (unsigned char) I2CMasterDataGet(ulI2CBase);
            }
            buf++;
         }
     }
     return 0;
}
