/*
  lcd.h - LCD Extended  declarations for the Micromint Eagle (Cortex-M3)
          by Micromint Support <support@micromint.com>    Mar-2009
*/

#ifndef _LCD_H
#define _LDC_H

#if defined(OPT_DEBUG_ENABLE) && \
    (!defined(LCD_XIO) && !defined(OPT_LCD_ANSI) && !defined(OPT_LCD_PORT))
#error Cannot use PC0-PC3 when debugger is active
#endif

#ifdef BOARD_EAGLE100
/* ----- Use CPLD for LCD I/O ----- */
#include "xio.h"
#define LCD_XIO

/* LCD declarations */
#define LCD_PORT                  XIO_PXD /* LCD data port */
#define LCD_CONTROL               XIO_PXE /* LCD control port */
#define LCD_RS                    0x08 /* Register select 1=Data,0=Cmd */
#define LCD_RW                    0x04 /* 1=Read,0=Write */
#define LCD_E                     0x02 /* 1=Enable,0=Disable */
#define LCD_BL                    0x01 /* Backlight 1=On,0=Off */
#elif defined(OPT_LCD_ANSI)
/* ----- Use ANSI terminal for LCD I/O ----- */
#include "ansi.h"
#define LCD_E                     0x02 /* 1=Enable,0=Disable */
#define LCD_BL                    0x01 /* Backlight 1=On,0=Off */
#else
/* ----- Use GPIO for LCD I/O ----- */
#include "jtag.h"
#include "inc/hw_gpio.h"
#include "driverlib/gpio.h"
#include "driverlib/sysctl.h"

/* LCD declarations */
#ifdef OPT_LCD_PORT
#define LCD_PORT                  GPIO_PORTF_BASE /* Alternate LCD data port */
#define LCD_PORT_SYSCTL           SYSCTL_PERIPH_GPIOF
#define LCD_CONTROL2              GPIO_PORTG_BASE /* LCD control port 2 */
#define LCD_CONTROL2_SYSCTL       SYSCTL_PERIPH_GPIOG
#define LCD_RS                    GPIO_PIN_2 /* Register select 1=Data,0=Cmd */
#define LCD_RW                    GPIO_PIN_3 /* 1=Read,0=Write */
#else
#define LCD_PORT                  GPIO_PORTC_BASE /* Default LCD data port */
#define LCD_PORT_SYSCTL           SYSCTL_PERIPH_GPIOC
#define LCD_CONTROL2              GPIO_PORTF_BASE /* LCD control port 2 */
#define LCD_CONTROL2_SYSCTL       SYSCTL_PERIPH_GPIOF
#define LCD_RS                    GPIO_PIN_0 /* Register select 1=Data,0=Cmd */
#define LCD_RW                    GPIO_PIN_1 /* 1=Read,0=Write */
#endif /* OPT_LCD_PORT */
#define LCD_PORT_MASK             0xff
#define LCD_CONTROL2_MASK         (LCD_RS | LCD_RW)
#define LCD_CONTROL               GPIO_PORTD_BASE /* LCD control port */
#define LCD_CONTROL_SYSCTL        SYSCTL_PERIPH_GPIOD
#define LCD_E                     GPIO_PIN_0 /* 1=Enable,0=Disable */
#define LCD_BL                    GPIO_PIN_1 /* Backlight 1=On,0=Off */
#define LCD_CONTROL_MASK          (LCD_E | LCD_BL)
#endif

/* Default to backlight on */
#define LCD_CTL_INIT              LCD_BL
/* Display options: default to cursor off */
#define LCD_DISP_ON               0x04
#define LCD_DISP_CURSOR           0x02
#define LCD_DISP_BLINK            0x01
#define LCD_DISP_INIT             (0x08 | LCD_DISP_ON)

/* API function declarations */

/* -------------------- LCD functions -------------------- */
void LcdInit(void);
unsigned long LcdCommand(unsigned long);
unsigned long LcdWrite(unsigned long);
unsigned long LcdWriteByte(unsigned long, unsigned long);
unsigned long LcdWriteStr(char*);
unsigned long LcdGoTo(unsigned long, unsigned long);
unsigned long LcdWriteAt(unsigned long, unsigned long, char*);
unsigned long LcdClear(void);
unsigned long LcdDisplay(unsigned long);
unsigned long LcdCursor(unsigned long);
unsigned long LcdBacklight(unsigned long);

#endif /* _LCD_H */
