/*
  rtc.h - Real-Time Clock Calendar Extended  declarations for the Micromint Eagle (Cortex-M3)
          by Micromint Support <support@micromint.com>    Nov-2009
*/
#ifndef _RTCC_H
#define _RTCC_H

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "inc/hw_gpio.h"
#include "inc/hw_i2c.h"
#include "driverlib/gpio.h"
#include "driverlib/i2c.h"
#include "driverlib/sysctl.h"
#include "bcd.h"
#include "i2c_rdwr.h"

/* RTC declarations */
#define RTC_ADDR        0x30
#define CMD_STATUS1     (RTC_ADDR | 0)
#define CMD_STATUS2     (RTC_ADDR | 1)
#define CMD_TIME1       (RTC_ADDR | 2)
#define CMD_TIME2       (RTC_ADDR | 3)
#define CMD_INT1        (RTC_ADDR | 4)
#define CMD_CCRA        (RTC_ADDR | 6)
#define CMD_FRA         (RTC_ADDR | 7)

#define BYTE_YEAR       0
#define BYTE_MONTH      1
#define BYTE_DAY        2
#define BYTE_WDAY       3
#define BYTE_HOURS      4
#define BYTE_MINS       5
#define BYTE_SECS       6

#define BYTE_ALWDAY     0
#define BYTE_ALHOUR     1
#define BYTE_ALMIN      2

#define FLAG_RESET      0x80
#define FLAG_24H        0x40 /* 1=24 hour mode, 0=12 hour mode */
#define FLAG_SCO        0x20
#define FLAG_SC1        0x10
#define FLAG_INT1       0x08
#define FLAG_INT2       0x04
#define FLAG_BLD        0x02
#define FLAG_POC        0x01

#define FLAG_INT1FE     0x80
#define FLAG_INT1ME     0x40
#define FLAG_INT1AE     0x20 /* 1=Alarm Enabled,0=Alarm Disabled */
#define FLAG_32kE       0x10
#define FLAG_INT2FE     0x08
#define FLAG_INT2ME     0x04
#define FLAG_INT2AE     0x02 /* 1=Alarm Enabled,0=Alarm Disabled */
#define FLAG_TEST       0x01


#define FLAG_AMPM       0x02   /* 1=PM,0=AM */
#define FLAG_AE         0x01   /* 1= alarm enabled, 0= alarm disabled */

#define AL_PERMIN       0 /* The alarm will go off every minute */
#define AL_WEEKLY       1 /* The alarm will go off the same day and time every week */
#define AL_DAILY        2 /* The alarm will go off every day at the same time */
#define AL_HOURLY       3 /* The alarm will go off the same time every hour */

/* RTC I/O */
#define RTC_PORT             GPIO_PORTB_BASE /* I2C clock and data */
#define RTC_PORT_SYSCTL      SYSCTL_PERIPH_GPIOB
#define RTC_CLK              GPIO_PIN_2
#define RTC_DATA             GPIO_PIN_3
#define RTC_I2C_BUS_IO       (RTC_CLK | RTC_DATA)
#define RTC_I2C_BUS          I2C0_MASTER_BASE
#define RTC_I2C_BUS_SYSCTL   SYSCTL_PERIPH_I2C0

struct rtc_time
{
         int tm_sec;
         int tm_min;
         int tm_hour;
         int tm_mday;
         int tm_mon;
         int tm_year;
         int tm_wday;
         int tm_yday;
         int tm_ampm;
};

/* API function declarations */
unsigned long RtcInit(void);
unsigned long RtcRdRegister(unsigned char *data, unsigned char reg);
unsigned long RtcWrRegister(unsigned char *data, unsigned char reg);
unsigned long RtcSetDateTime(struct rtc_time *tm);
unsigned long RtcReadDateTime(struct rtc_time *tm);
unsigned long RtcChngTimeFormat(void);
unsigned long RtcTimeFormat(void);
unsigned long RtcSetAlTime(unsigned long occur, struct rtc_time *altm);
unsigned long RtcReadAlTime(struct rtc_time *altm);
unsigned long RtcEnableAlarm(unsigned long mode);
unsigned long RtcDisableAlarm(void);
#endif /* _RTCC_H */
