/*
  xio.h - Extended I/O declarations for the Micromint Eagle SBCs (Cortex-M3)
          by Micromint Support <support@micromint.com>    Jul-2008
*/

#ifndef _XIO_H
#define _XIO_H

#if !defined(BOARD_EAGLE100)
#warning "Extended I/O only available on boards with programmable logic"
#define NO_XIO
#else

#include "inc/hw_gpio.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "driverlib/gpio.h"
#include "driverlib/ssi.h"
#include "driverlib/sysctl.h"

/* SSI port */
#define XIO_SSI_BASE              SSI0_BASE
#define XIO_SSI_SYSCTL_PERIPH     SYSCTL_PERIPH_SSI0

/* GPIO for SSI pins - PA2/SSI0CLK, PA4/SSI0RX, PA5/SSI0TX */
#define XIO_GPIO_PORT_BASE        GPIO_PORTA_BASE
#define XIO_GPIO_SYSCTL_PERIPH    SYSCTL_PERIPH_GPIOA
#define XIO_SSI_CLK               GPIO_PIN_2
#define XIO_SSI_TX                GPIO_PIN_5
#define XIO_SSI_RX                GPIO_PIN_4
#define XIO_SSI_PINS              (XIO_SSI_TX | XIO_SSI_RX | XIO_SSI_CLK)

/* GPIO for CPLD chip select - PA3 */
#define XIO_CS_GPIO_PORT_BASE     GPIO_PORTA_BASE
#define XIO_CS_GPIO_SYSCTL_PERIPH SYSCTL_PERIPH_GPIOA
#define XIO_CS                    GPIO_PIN_3

/*
   9-bit frame format

   Bit8 = 1 (command)
   Bit  7 - 4     3 - 0
        xxxx      yyyy
        4-bit     4-bit
        opcode    port

   Bit8 = 0 (data)
   Bit  7  -   0
        zzzzzzzz
        8-bit
        data
*/

/* Port and register declarations */
#define XIO_PORT_NONE             0x00
#define XIO_PXA                   0x01
#define XIO_PXB                   0x02
#define XIO_PXC                   0x03
#define XIO_PXD                   0x04
#define XIO_PXE                   0x05
#define XIO_PXF                   0x06
#define XIO_PXG                   0x07
#define XIO_PXH                   0x08
#define XIO_PXI                   0x09
#define XIO_PXJ                   0x0a
#define XIO_PORT_MASK             0x0f

#define XIO_RXA                   0x0b
#define XIO_RXB                   0x0c
#define XIO_RXC                   0x0d
#define XIO_RXD                   0x0e

/* Opcode declarations */
#define XIO_CMD_NONE              0x00
#define XIO_CMD_READ              0x10
#define XIO_CMD_WRITE             0x20
#define XIO_CMD_MASK              0xf0

#define XIO_CMD_SEL               0x0100
#define XIO_DATA_MASK             0x00ff

/* ISA declarations */
#define ISA_PORT                  XIO_PXF /* ISA bus data port */
#define ISA_ADDRL                 XIO_PXG /* ISA bus low address port */
#define ISA_ADDRH                 XIO_PXH /* ISA bus high address port */
#define ISA_LATCH                 XIO_PXI /* ISA bus latch */
#define ISA_STATUS                XIO_PXJ /* ISA bus status */
#define ISA_DATA_MASK             XIO_DATA_MASK

/* API function declarations */

/* -------------------- Extended I/O functions -------------------- */
void XioInit(void);
unsigned long XioWrite(unsigned long, unsigned long);
unsigned long XioWriteBlock(unsigned long, unsigned long*, unsigned long);
unsigned long XioRead(unsigned long);
unsigned long XioReadBlock(unsigned long, unsigned long*, unsigned long);

/* -------------------- ISA bus functions -------------------- */
void IsaInit(void);
unsigned long IsaWrite(unsigned long, unsigned long);
unsigned long IsaWriteBlock(unsigned long, unsigned long*, unsigned long);
unsigned long IsaRead(unsigned long);
unsigned long IsaReadBlock(unsigned long, unsigned long*, unsigned long);

#endif /* NO_XIO */

#endif /* _XIO_H */
